from __future__ import annotations

from enum import Enum


class ReasonCode(str, Enum):
    SEARCH_FIRST_REQUIRED = "SEARCH_FIRST_REQUIRED"
    SEARCH_REF_REQUIRED = "SEARCH_REF_REQUIRED"
    CANDIDATE_REF_REQUIRED = "CANDIDATE_REF_REQUIRED"
    BUDGET_SOFT_LIMIT = "BUDGET_SOFT_LIMIT"
    BUDGET_HARD_LIMIT = "BUDGET_HARD_LIMIT"
    LOW_RELEVANCE_OUTSIDE_TOPK = "LOW_RELEVANCE_OUTSIDE_TOPK"
    PREVIEW_DEGRADED = "PREVIEW_DEGRADED"
